 	program sea_elastic
c
c This program solves the sea level equation for a one step elastic fingerprint
c in the case where the system is evolved forward from present-day and shoreline 
c migration is not included.
c
c ......................................................
c
c Three parameters will be specified in parameter statements
c   nspha  - is the size of the spherical harmonic coefficient vectors
c            (this number must be consistent with alpol.f)
c   norder - is the spherical harmonic truncation level
c   narray - the total number of grid points E-W
c
c ........................................................
c
	parameter(norder=512)
	parameter(nspha=134000)
	parameter(narray=2*norder)
c
c Declarations ....
c
c Quantities used for the impulse response parameters
	dimension xf(norder),ep(norder)
	dimension xff(nspha),epp(nspha)
	dimension ep2(norder)
	dimension epp2(nspha)
c
c Quantities used in the matrix solutions of the sealevel eqn.
	complex*8 elrv(nspha),elrv2(nspha)
	complex*8 zice(nspha),zsea(nspha)
	complex*8 ocean(nspha),sum1
        complex*8 vec1(nspha),vec2(nspha),vec3(nspha),vec4(nspha)
        complex*8 vec1b(nspha),vec3b(nspha)
	real summ(200)
        dimension ob(norder,narray),tinit(norder,narray)
        dimension slgrid(norder,narray)
        dimension ggrid(norder,narray)
c
c Qauntities used for input ice grid
        real ice_start(norder,narray),ice_end(norder,narray)
        real ice_diff(norder,narray)
c
c Legendre polynomial array ...
	real pmn(nspha)
c Gauss legendre grid and weight vectors
	real*8 glat(norder),gwt(norder),x
c
c .........................................................
c Some common statements
c
	common pmn
	common/vint/ ino,inom,ino2,nlat,inlat,itot
c
c Set up some array size constants ...
c
        inom=norder
        ino=norder+1
	ino2=2*ino
c
	nlat=norder
	inlat=nlat/2
c
	deg=float(norder)+1.5
c
c Total file size will govern the dimension of the matrices 
        ict=mn(inom,ino,deg)
        itot=ict
	if(itot.gt.nspha) then
	 write(6,*) 'declared array dimension is too small'
	 stop
         endif
c
c Set up some constants ........
c
	pi=3.1415926535898
	erad=6.371e6
	emass=5.976e24
        rhoi=920.0
	rhow=1000.0
	tau=4.0*pi*(erad**3.0)/emass
c
c ...........................................................
c      zero out some vectors
        do i=1,norder
         xf(i)=0.0
         ep(i)=0.0
         ep2(i)=0.0
         enddo
        do i=1,nspha
          epp(i)=0.0
          epp2(i)=0.0
          xff(i)=0.0
          elrv(i)=(0.,0.)
          elrv2(i)=(0.,0.)
          zice(i)=(0.,0.)
          zsea(i)=(0.,0.)
          ocean(i)=(0.,0.)
          vec1(i)=(0.,0.)
          vec1b(i)=(0.,0.)
          vec2(i)=(0.,0.)
          vec3(i)=(0.,0.)
          vec3b(i)=(0.,0.)
          vec4(i)=(0.,0.)
          enddo
c
c The tolerance for convergence of the iterative sea-level scheme
        ztol=1.0e-4
c
c Set up the output file for the next sea level increment
        call askfil(13,' output - sea level harmonics')
	call askfil(14,' output - sea level grid')
c
c ............................................................
c Read in the gauss-legendre weights and latitudes
c
	call askfil(10,' gauss-leg. latitudes ')
	call askfil(11,' gauss-leg. weights   ')
	read(10,*) (glat(j),j=1,inlat)
	read(11,*) (gwt(j),j=1,inlat)
        close(10)
        close(11)
c 
c The output of alpol provides the Legendre polynomial for each 
c colatitude in the solution - reading these in at the beginning is
c faster than computing them within the sea level code
c
        ialpol=88
	open(ialpol,
     .   file='SPECTRAL/alpol.512',form='unformatted',status='old')
c
c ...........................................................
c
c Read in ice thicknesses before and after loading
c
        call askfil(10,'ice before ')
        call askfil(11,'ice after ')
        read(10,*) ((ice_start(i,j),j=1,narray),i=1,norder)
        read(11,*) ((ice_end(i,j),j=1,narray),i=1,norder)
        close(10)
        close(11)
c
c Compute the difference in these, which will constitute the load
c
        do i=1,norder
          do j=1,narray
            ice_diff(i,j)=ice_end(i,j)-ice_start(i,j)
            enddo
          enddo
c
c
        call askfil(9,' topo ')
        read(9,*) ((tinit(i,j),j=1,narray),i=1,norder)
        close(9)
c
c Use the topography to compute the ocean function
c (1 where topography is less than or equal to zero, 0 elsewhere)
c
        do i=1,norder
          do j=1,narray
          if(tinit(i,j).gt.0) then
            ob(i,j)=0.0
          else
            ob(i,j)=1.0
          endif
          enddo
         enddo
c
c Convert ob and ice_diff grid to spherical harmonics
c
	 call convert(ob,ice_diff,ialpol,glat,gwt,ocean,zice)
c
c Check of total file size 
c
        ict=mn(inom,ino,deg)
        itot=ict
	if(itot.gt.nspha) then
	 write(6,*) 'declared array dimension is too small'
	 stop
         endif
c
c .........
c
c Zero out the ocean load harmonics (i.e., these are being solved for)
c
	do i=1,itot
	 zsea(i)=(0.,0.)
         enddo
c
c ..............................................
c
c Next we read in elastic Love number combos (1+k-h) (and check that the 
c file is complete)
c
	call askfil(2,' data from maxwell ')
        do i=1,norder
          read(2,*) jjj,ep(i),ep2(i)
          if(jjj.ne.i) then
            write(6,*) 'love number file is not complete '
            stop
            endif
          enddo
c
c ........................................................
c
c Calculate the degree dependent dimensional scaling factor
c
	do i=1,norder
	 xf(i)=(1.0/(2.0*float(i)+1.0))*tau
         enddo
c
c The arrays ep and xf have "norder" values - each element
c refers to a degree and they are not dependent on harmonic order
c For convenience, we onvert ep and xf into a full (m,n) array
c
	epp(1)=0.0
	epp2(1)=0.0
	xff(1)=0.0
	do87 m=1,ino
	 do88 n=m,ino
	  k1=mn(m-1,n-1,deg)
	  if(k1.eq.1) go to 88
	  epp(k1)=ep(n-1)
	  epp2(k1)=ep2(n-1)
	  xff(k1)=xf(n-1)
 88	  continue
 87	 continue
c
c ..............................................
c
c The Elastic Response Vector
c
	do n=1,itot
	 elrv(n)=epp(n)*zice(n)*rhoi
	 elrv2(n)=epp2(n)*zice(n)*rhoi
         enddo
c
c ..................................................................
c
c The total response so far ...
c
	do101 m=1,ino
	 do102 n=m,ino
	  k1=mn(m-1,n-1,deg)
	  if(k1.eq.1) go to 102
	  vec1(k1)=xff(k1)*elrv(k1)
	  vec1b(k1)=xff(k1)*elrv2(k1)
 102	 continue
 101	continue
c
c ..............................................
c
c Start the sea-level iteration procedure!
c
c Step 1: the first guess to the ocean load change is the eustatic change
c
c note: summ is a parameter used to check the iteration convergence
c
	summ(1)=0.0
	zzfe=real(-rhoi*zice(1)/(rhow*ocean(1)))
	do220 m=1,ino
	 do320 n=m,ino
	 k1=mn(m-1,n-1,deg)
	 vec2(k1)=zzfe*ocean(k1)
	 summ(1)=summ(1)+cabs(vec2(k1))
 320	 continue
 220	continue
	if(summ(1) .lt. 1.e-6) summ(1)=1.e-6
c
c
c Step 2: begin the sea-level iterations!
c
c The sea level algorithm seeks to iteratively improve the sea level load
c (the first guess for which - as above - is the eustatic change). 
c
        itermax=10
	do240 niter=1,itermax
	 inp=niter+1
	 summ(inp)=0.0
c
c Set up the total sea level (minus dc) esponse vector so far ...
c
	 do221 m=1,ino
	  do321 n=m,ino
	  k1=mn(m-1,n-1,deg)
	  if(k1.eq.1) go to 321
	   vec3(k1)=vec1(k1)+xff(k1)*epp(k1)*vec2(k1)*rhow
	   vec3b(k1)=vec1b(k1)+xff(k1)*epp2(k1)*vec2(k1)*rhow
 321	  continue
 221	 continue
c
c
c
c First, calculate the total response (minus dc term) and ocean functions 
c on a grid, multiply them and then compute the harmonic decomposition of 
c the product ...
c
	 call combin(vec3,ocean,ialpol,glat,gwt,vec4)
c
c Second, calculate the conservation of mass component
c
	 sum1=(-(rhoi/rhow)*zice(1)-vec4(1))/ocean(1)
c
c Third, update the ocean load vector ...
c
	 do250 m=1,ino
	  do350 n=m,ino
	  k1=mn(m-1,n-1,deg)
	  vec2(k1)=vec4(k1)+sum1*ocean(k1)
	  summ(inp)=summ(inp)+cabs(vec2(k1))
 350	  continue
 250	  continue
c
c Compute difference in the norm of the sea level load across
c two iteration steps. If this difference is less than ztol,
c iteration stops
c
	 z=abs((summ(inp)-summ(niter))/summ(niter))
	 write(6,*) niter,sum1,z
	 if(z.lt.ztol) go to 260
c
 240	 continue
c
 260 	continue
c 
c Out of the iteration!
c
c Step 3: Write out the ocean load vector ...
c
c       do103 m=1,ino
c        do104 n=m,ino,2
c         k1=mn(m-1,n-1,deg)
c         k2=mn(m-1,n,deg)
c         vr1=real(vec2(k1))
c         vi1=aimag(vec2(k1))
c         vr2=real(vec2(k2))
c         vi2=aimag(vec2(k2))
c         write(4,921) vr1,vi1,vr2,vi2
c104	  continue
c103	 continue
c        write(4,*) sum1
c
 921    format(e14.7,1x,e14.7,4x,e14.7,1x,e14.7)
c
c Step 4: Write out the global sea level vector ...
c
        vec3(1)=sum1
        vec3b(1)=sum1
        write(13,*) inom,0.0
        do803 m=1,ino
          do804 n=m,ino,2
          k1=mn(m-1,n-1,deg)
          k2=mn(m-1,n,deg)
          vr1=real(vec3(k1))
          vi1=aimag(vec3(k1))
          vr2=real(vec3(k2))
          vi2=aimag(vec3(k2))
          write(13,921) vr1,vi1,vr2,vi2
 804	  continue
 803	 continue
         call convert2(vec3,ialpol,glat,gwt,slgrid)
         call convert2(vec3b,ialpol,glat,gwt,ggrid)
c
c Step 5: Write out the ocean height and global sea level grid
c 
        do i=1,norder
c          write(14,*) (slgrid(i,j),j=1,narray)
           write(14,*) (ggrid(i,j),j=1,narray)
           enddo 
c
	stop
	end
c
	function efjn(tmp1,tmp2)
	alf=0.0
	if(tmp2.le.160.)alf=exp(-tmp2)
	alfa=1.0-alf
	efjn=alfa*tmp1
	return
	end
c
	subroutine combin(trv,ocean,ialpol,glat,gwt,vmn)
c this program takes two mn files, synthesizes the respective
c fields, multiplies them, and then decomposes the product field
	parameter(nspha=134000)
	parameter(norder=512)
	parameter(narray=2*norder)
c       real alpol(nspha,norder)
        real data(narray,norder),pmn(nspha)
	complex trv(nspha),ocean(nspha)
	complex vm(narray),vmn(nspha),vtheta(narray)
	complex temp(5000),temp2(5000)
	real*8 glat(norder),gwt(norder),wt
	common pmn
	common/vint/ino,inom,ino2,nlat,inlat,itot
	deg=float(inom)+1.5
c evaluate the matrix of functional values ...
        rewind(ialpol)
	do11 i=1,itot
	 vmn(i)=(0.,0.)
 11	 continue
	do30 j=1,nlat
	call readr(ialpol,pmn,nspha)
	do 34 m=1,ino
	temp(m)=0.
	temp2(m)=0.
	do 35 n=m,ino
	k1=mn(m-1,n-1,deg)
	temp(m)=temp(m)+trv(k1)*pmn(k1)       
 	temp2(m)=temp2(m)+ocean(k1)*pmn(k1)      
 35	continue
 34	continue
	do 36 m=2,inom
        temp(ino2-m)=conjg(temp(m))
 36	temp2(ino2-m)=conjg(temp2(m))
	call fouriv (temp,narray,1.0)
	call fouriv(temp2,narray,1.0)
c
	do37 i=1,narray
	 s=(temp(i)+conjg(temp(i)))*0.5
	 s2=(temp2(i)+conjg(temp2(i)))*0.5
 37	 data(i,j)=real(s)*real(s2)
c  37	data(i,j)=real(s)
 30	 continue
c ..........................................
c next we have to decompose the field ...
c
        rewind(ialpol)
	do 40 k=1,nlat
	call readr(ialpol,pmn,nspha)
	do 47 j=1,narray
 47	vtheta(j)=data(j,k)
	call fouriv(vtheta,narray,-1.)
	do 48 m=1,ino
 48	vm(m)=vtheta(m)/float(narray)
	i=1
	if(k.gt.inlat)i=-1
	wt=(1+i)/2*gwt(k*(1+i)/2+(1-i)/2)+(1-i)/2*gwt((nlat-k+1)*(1-i)/2
     .+(1+i)/2)
	do 53 m=1,ino
	do 54 n=m,ino
        vmn(mn(m-1,n-1,deg))=vmn(mn(m-1,n-1,deg))+0.5*wt*vm(m)*pmn
     .(mn(m-1,n-1,deg))
 54	continue
 53	continue
c
 40	continue
c ................................................
	return
	end
c
	subroutine convert(data,data2,ialpol,glat,gwt,vmn,vmn2)
c this program takes a gl-grid and converts to a spherical harmonic file
	parameter(nspha=134000)
	parameter(norder=512)
	parameter(narray=2*norder)
        real data(norder,narray),pmn(nspha)
        real data2(norder,narray)
	complex vm(narray),vmn(nspha),vtheta(narray)
	complex vm2(narray),vmn2(nspha),vtheta2(narray)
	real*8 glat(norder),gwt(norder),wt
	common pmn
	common/vint/ino,inom,ino2,nlat,inlat,itot
	deg=float(inom)+1.5
        rewind(ialpol)
	do11 i=1,nspha
	 vmn(i)=(0.,0.)
	 vmn2(i)=(0.,0.)
 11	 continue
c
c decompose the field ...
c
        rewind(ialpol)
	do 40 k=1,nlat
	call readr(ialpol,pmn,nspha)
	do 47 j=1,narray
   	vtheta2(j)=data2(k,j)
 47	vtheta(j)=data(k,j)
	call fouriv(vtheta,narray,-1.)
	call fouriv(vtheta2,narray,-1.)
	do 48 m=1,ino
   	vm2(m)=vtheta2(m)/float(narray)
 48	vm(m)=vtheta(m)/float(narray)
	i=1
	if(k.gt.inlat)i=-1
	wt=(1+i)/2*gwt(k*(1+i)/2+(1-i)/2)+(1-i)/2*gwt((nlat-k+1)*(1-i)/2
     .+(1+i)/2)
	do 53 m=1,ino
	do 54 n=m,ino
        vmn2(mn(m-1,n-1,deg))=vmn2(mn(m-1,n-1,deg))+0.5*wt*vm2(m)*pmn
     .(mn(m-1,n-1,deg))
        vmn(mn(m-1,n-1,deg))=vmn(mn(m-1,n-1,deg))+0.5*wt*vm(m)*pmn
     .(mn(m-1,n-1,deg))
 54	continue
 53	continue
c
 40	continue
c ................................................
	return
	end
c
	subroutine convert2(trv,ialpol,glat,gwt,data)
c this program takes an mn file and converts to a gl-grid
	parameter(nspha=134000)
	parameter(norder=512)
	parameter(narray=2*norder)
        real data(norder,narray),pmn(nspha)
	complex trv(nspha)
	complex temp(5000)
	common pmn
	common/vint/ino,inom,ino2,nlat,inlat,itot
	deg=float(inom)+1.5
c evaluate the matrix of functional values ...
        rewind(ialpol)
	do30 j=1,nlat
	call readr(ialpol,pmn,nspha)
	do 34 m=1,ino
	temp(m)=0.
	do 35 n=m,ino
	k1=mn(m-1,n-1,deg)
	temp(m)=temp(m)+trv(k1)*pmn(k1)       
 35	continue
 34	continue
	do 36 m=2,inom
 36     temp(ino2-m)=conjg(temp(m))
	call fouriv (temp,narray,1.0)
c
	do37 i=1,narray
	 s=(temp(i)+conjg(temp(i)))*0.5
 37	 data(j,i)=real(s)
 30	 continue
c ..........................................
	return
	end
c
	function mn(m,n,deg)
	real deg
	integer m,n
	mn=ifix(deg*m-m*m*0.5+n+0.5)+1
	return
	end
c
	function imn(m,n,deg)
	real deg
	integer m,n
	imn=ifix(deg*m-m*m*0.5+n+0.5)+1
	return
	end
c
	subroutine readr(ichan,pmn,nspha)
	real pmn(nspha)
	read(ichan) pmn
	return
	end

